package no.integration.svv.config;

import no.integration.svv.properties.WSClientConfigProperties;
import org.apache.http.Header;
import org.apache.http.client.HttpClient;
import org.apache.http.client.protocol.RequestDefaultHeaders;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;
import org.springframework.boot.webservices.client.WebServiceTemplateBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.oxm.jaxb.Jaxb2Marshaller;
import org.springframework.ws.WebServiceMessageFactory;
import org.springframework.ws.client.core.WebServiceTemplate;
import org.springframework.ws.soap.saaj.SaajSoapMessageFactory;
import org.springframework.ws.transport.http.HttpComponentsMessageSender;

import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPConstants;
import javax.xml.soap.SOAPException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;
/**
 @author vijayjadhav
 **/

/**
 * Contains overall application config of Webservice Template,
 * HttpMessage sender, Httpclient, Marshaller
 */

@Configuration
public class ApplicationConfig {


    @Bean
    public WSClientConfigProperties wsClientConfigProperties() {
        return new WSClientConfigProperties();
    }

    @Bean
    public Jaxb2Marshaller marshaller() {
        Jaxb2Marshaller marshaller = new Jaxb2Marshaller();
        marshaller.setPackagesToScan(this.wsClientConfigProperties().getScanPackages());
        return marshaller;
    }

    /**
     * Creates HTTP client connection instance with props to authenticate access to the MimeWS server
    * */
    @Bean
    public HttpClient httpClient() {
        List<Header> headers = new ArrayList<>();
        BasicHeader authHeader = new BasicHeader("Authorization", "Basic " + wsClientConfigProperties().getSecurityKey());
        headers.add(authHeader);

        RequestDefaultHeaders reqHeader = new RequestDefaultHeaders(headers);

        return HttpClients.custom()
                        .addInterceptorFirst(new HttpComponentsMessageSender.RemoveSoapHeadersInterceptor())
                        .addInterceptorLast(reqHeader)
                        .build();

    }

    /**
     * Allows to use the configured HttpClient instance above with authentication to execute request
    * */
    @Bean
    public HttpComponentsMessageSender defaultMyMessageSender()
            throws KeyManagementException, NoSuchAlgorithmException, KeyStoreException {
        HttpComponentsMessageSender messageSender = new HttpComponentsMessageSender(httpClient());
        return messageSender;
    }

    /**
     * Marshalling and unmarshalling of the objects from Su-Fa - integration - MimeWS server
     * Sends a web service message that contains the given payload(marshalled)
     * Returns the unmarshalled payload of the response message
    * */
    @Bean
    public WebServiceTemplate webServiceTemplate(WebServiceTemplateBuilder builder) throws SOAPException, NoSuchAlgorithmException, KeyStoreException, KeyManagementException {
        MessageFactory mf = MessageFactory.newInstance(SOAPConstants.SOAP_1_2_PROTOCOL);
        WebServiceMessageFactory webMessageFactory = new SaajSoapMessageFactory(mf);

        return builder.messageSenders(defaultMyMessageSender())
                .setMarshaller(marshaller())
                .setUnmarshaller(marshaller())
                .setDefaultUri(this.wsClientConfigProperties().getLocationURI())
                .setWebServiceMessageFactory(webMessageFactory)
                .build();

    }

}
