package no.integration.svv.config;

import lombok.Data;
import no.integration.svv.properties.AuthConfigProperties;
import no.integration.svv.utils.authentication.Authentication;
import no.integration.svv.utils.authentication.MissingTokenException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.server.EndpointInterceptor;
import org.springframework.ws.transport.context.TransportContext;
import org.springframework.ws.transport.context.TransportContextHolder;
import org.springframework.ws.transport.http.HttpServletConnection;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

import static no.integration.svv.utils.authentication.AuthenticationConstants.AUTH_TOKEN_EMPTY_LOGTEKST;
import static no.integration.svv.utils.authentication.AuthenticationConstants.AUTH_TOKEN_MISSING_LOGTEKST;
/**
 @author vijayjadhav
 **/

/**
Contains Auth2 token validation interceptor that utilises
the Authentication steps to validate Key and Value provided in the request payload
*/

@Data
@ConfigurationProperties(prefix = "authentication")
public class AuthenticationInterceptor implements EndpointInterceptor {

    private static final Logger log = LoggerFactory.getLogger(AuthenticationInterceptor.class);

    private final AuthConfigProperties authConfigProperties;
    private final List<String> roles = new ArrayList<>();

    public AuthenticationInterceptor(AuthConfigProperties authConfigProperties) {

        roles.add(authConfigProperties.getRoles());
        this.authConfigProperties = authConfigProperties;

    }

    /**
     * Method to authenticate and validate the OAuth2 token provided in the request from Su-Fa
     * */
    @Override
    public boolean handleRequest(MessageContext messageContext, Object endpoint) throws Exception {
        log.info("Authentication Interceptor::handleRequest - Start");
        AtomicReference<String> headerValue = new AtomicReference<>();

        TransportContext transportContext = TransportContextHolder.getTransportContext();
        HttpServletConnection connection = (HttpServletConnection) transportContext.getConnection();
        Iterator<String> requestHeaders = connection.getRequestHeaders(authConfigProperties.getHeaderKey());
        requestHeaders.forEachRemaining(headerVal -> {
            if (headerVal != null && !headerVal.isEmpty()) {
                headerValue.set(headerVal);
            }
        });

        if (headerValue == null) {
            log.error(AUTH_TOKEN_MISSING_LOGTEKST);
            throw new MissingTokenException(AUTH_TOKEN_MISSING_LOGTEKST);
        }
        String authorization = headerValue.get();
        if (authorization.isEmpty()) {
            log.error(AUTH_TOKEN_EMPTY_LOGTEKST);
            throw new MissingTokenException(AUTH_TOKEN_EMPTY_LOGTEKST);
        }
        log.info("Authentication Interceptor::handleRequest - Retrieved authorization token");
        String jwsToken = authorization.substring(authorization.indexOf(" ") + 1);

        new Authentication().validate(jwsToken, this.roles, authConfigProperties.getJwks(), authConfigProperties.getOidc_issuer());
        log.info("Authentication Interceptor::handleRequest - End");
        return true;
    }


    @Override
    public boolean handleResponse(MessageContext messageContext, Object endpoint) throws Exception {
        log.info("Response handling");
        return true;
    }

    @Override
    public boolean handleFault(MessageContext messageContext, Object endpoint) throws Exception {
        log.info("Exception handling");
        return true;
    }

    @Override
    public void afterCompletion(MessageContext messageContext, Object endpoint, Exception ex) throws Exception {
        log.info("Request completed!");

    }


}
