package no.integration.svv.endpoint;

import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.*;
import no.integration.svv.exception.DataMappingException;
import no.integration.svv.exception.ValidationException;
import no.integration.svv.service.SoapClient;
import no.integration.svv.utils.mapper.DokumentMap;
import no.integration.svv.utils.mapper.JournalPost;
import no.integration.svv.utils.mapper.OppJournalpostStatus;
import no.integration.svv.utils.mapper.SaksMappe;
import no.integration.svv.utils.validator.RequestValidator;
import no.integration.svv.utils.validator.ResponseValidator;
import no.vegvesen.saksarkiv.felles.typer._1.JournalpostKvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.KvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.MappeKvitteringType;
import no.vegvesen.saksarkiv.felles.typer._1.NyttVedleggKvitteringType;
import no.vegvesen.saksarkiv.journalpost.melding.endrestatus._1.EndreStatusRequestType;
import no.vegvesen.saksarkiv.journalpost.melding.leggtilnytt._1.LeggTilNyttDokumentJournalpostRequestType;
import no.vegvesen.saksarkiv.journalpost.melding.opprett._1.OpprettJournalpostRequestType;
import no.vegvesen.saksarkiv.saksmappe.melding.opprett._1.OpprettSaksmappeRequestType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.ws.server.endpoint.annotation.Endpoint;
import org.springframework.ws.server.endpoint.annotation.PayloadRoot;
import org.springframework.ws.server.endpoint.annotation.RequestPayload;
import org.springframework.ws.server.endpoint.annotation.ResponsePayload;

import java.io.UnsupportedEncodingException;

/**
 @author vijayjadhav
 **/
/**
 * Processes all 4 operation request to the endpoint and provides response based on the mapping
* */
@Endpoint
public class GeoIntegrationEndpoint {

    private static final String NS = "http://rep.geointegrasjon.no/Arkiv/Oppdatering/xml.wsdl/2012.01.31";
    private static final String LOCALPART_NY_JOURNAL_POST = "NyJournalpost";
    private static final String LOCALPART_NY_SAKSMAPPE = "NySaksmappe";
    private static final String LOCALPART_NY_DOKUMENT = "NyDokument";
    private static final String LOCALPART_OPPDATER_JOURNALPOST_STATUS = "OppdaterJournalpostStatus";


    private static final Logger log = LoggerFactory.getLogger(GeoIntegrationEndpoint.class);
    private final SoapClient soapClient;


    public GeoIntegrationEndpoint(final SoapClient soapClient) {
        this.soapClient = soapClient;
    }

    /**
     * NyJournalPost operation request/response handling based on the mapping
     * */
    @PayloadRoot(namespace = NS, localPart = LOCALPART_NY_JOURNAL_POST)
    @ResponsePayload
    public NyJournalpostResponse nyJournalpostHandler(@RequestPayload NyJournalpost request) throws Exception {
        log.info("GeoIntegrationEndpoint::nyJournalpostHandler - Start");
        try {
            RequestValidator.validateJournalpostRequest(request);
            OpprettJournalpostRequestType mimeInput = JournalPost.endpointToMimeOppJPT(request);
            log.info("GeoIntegrationEndpoint::nyJournalpostHandler - Sent JournalPost input request to MimeWS");

            // call Mime 360  endpoint with mimeinput
            JournalpostKvitteringType mimeResponse = soapClient.opprettJournalPost(mimeInput);
            ResponseValidator.validateJournalPostResponse(mimeResponse);
            log.info("GeoIntegrationEndpoint::nyJournalpostHandler - End");
            return JournalPost.nyJournalpostResponseToSuFa(mimeResponse);
        } catch (ValidationException e) {
            log.error("nyJournalpostHandler - Validation error: Endpoint = {}, FieldName= {}, Message= {}, Stacktrace={}", e.getEndpointName(), e.getFieldName(), e.getMessage(), e.getStackTrace());
//            log.error(String.valueOf(SplunkLogging.log().med("Validation Error","nyJournalpostHandler")));
            throw new RuntimeException(e);
        } catch (DataMappingException e) {
            log.error("nyJournalpostHandler - Data mapping error: FieldName = {}, FieldValue= {}, Message= {}, Stacktrace={}", e.getFieldName(), e.getFieldValue(), e.getMessage(), e.getStackTrace());
//            log.error(String.valueOf(SplunkLogging.log().med("Data mapping error","nyJournalpostHandler")));
            throw new RuntimeException(e);
        }
    }

    /**
     * NySaksmappe operation request/response handling based on the mapping
     * */
    @PayloadRoot(namespace = NS, localPart = LOCALPART_NY_SAKSMAPPE)
    @ResponsePayload
    public NySaksmappeResponse nySaksmappeHandler(@RequestPayload NySaksmappe request) {
        log.info("GeoIntegrationEndpoint::nySaksmappeHandler - Start");
        try {
            RequestValidator.validateNySaksmappeRequest(request);
            OpprettSaksmappeRequestType mimeInput = SaksMappe.endpointToMimeOppSM(request);
            log.info("GeoIntegrationEndpoint::nySaksmappeHandler - Sent SaksMappe input request to MimeWS");
            MappeKvitteringType mimeResponse = soapClient.opprettSaksmappe(mimeInput);
            ResponseValidator.validateSaksmappeResponse(mimeResponse);
            log.info("GeoIntegrationEndpoint::nySaksmappeHandler - End");
            return SaksMappe.nySaksmappeResponseToSuFa(mimeResponse);

        } catch (ValidationException e) {
            log.error("nySaksmappeHandler - Validation error: Endpoint = {}, FieldName= {}, Message= {}, Stacktrace={}", e.getEndpointName(), e.getFieldName(), e.getMessage(), e.getStackTrace());
//            log.error(String.valueOf(SplunkLogging.log().med("Validation Error","nySaksmappeHandler")));
            throw new RuntimeException(e);
        } catch (DataMappingException e) {
            log.error("nySaksmappeHandler - Data mapping error: FieldName = {}, FieldValue= {}, Message= {}, Stacktrace={}", e.getFieldName(), e.getFieldValue(), e.getMessage(), e.getStackTrace());
//            log.error(String.valueOf(SplunkLogging.log().med("Data mapping error","nySaksmappeHandler")));
            throw new RuntimeException(e);
        }

    }

    /**
     * NyDokument operation request/response handling based on the mapping
     * */
    @PayloadRoot(namespace = NS, localPart = LOCALPART_NY_DOKUMENT)
    @ResponsePayload
    public NyDokumentResponse nyDokumentHandler(@RequestPayload NyDokument request) throws UnsupportedEncodingException {
        log.info("GeoIntegrationEndpoint::nyDokumentHandler - Start");
        try {
            RequestValidator.validateNyDokumentRequest(request);
            LeggTilNyttDokumentJournalpostRequestType mimeInput = DokumentMap.endpointToMimeOppDok(request);
            log.info("GeoIntegrationEndpoint::nyDokumentHandler - Sent Document input request to MimeWS");
            NyttVedleggKvitteringType mimeResponse = soapClient.opprettDokument(mimeInput);
            ResponseValidator.validateDokumentResponse(mimeResponse);
            log.info("GeoIntegrationEndpoint::nyDokumentHandler - End");
            return DokumentMap.nyDokumentResponseToSuFa(mimeResponse);

        } catch (ValidationException e) {
            log.error("nyDokumentHandler - Validation error: Endpoint = {}, FieldName= {}, Message= {}, Stacktrace={}", e.getEndpointName(), e.getFieldName(), e.getMessage(), e.getStackTrace());
//            log.error(String.valueOf(SplunkLogging.log().med("Validation Error","nyDokumentHandler")));
            throw new RuntimeException(e);
        } catch (DataMappingException e) {
            log.error("nyDokumentHandler - Data mapping error: FieldName = {}, FieldValue= {}, Message= {}, Stacktrace={}", e.getFieldName(), e.getFieldValue(), e.getMessage(), e.getStackTrace());
//            log.error(String.valueOf(SplunkLogging.log().med("Data mapping error","nyDokumentHandler")));
            throw new RuntimeException(e);
        }

    }

    /**
     * NyJournalPostStatus operation request/response handling based on the mapping
     * */
    @PayloadRoot(namespace = NS, localPart = LOCALPART_OPPDATER_JOURNALPOST_STATUS)
    @ResponsePayload
    public OppdaterJournalpostStatusResponse nyJPStatusHandler(@RequestPayload OppdaterJournalpostStatus request) {
        log.info("GeoIntegrationEndpoint::nyJPStatusHandler - Start");
        try {
            RequestValidator.validateJPStatusRequest(request);
            EndreStatusRequestType mimeInput = OppJournalpostStatus.endpointToMimeOppJPStatus(request);
            log.info("GeoIntegrationEndpoint::nyJPStatusHandler - Sent JournalPost Status input request to MimeWS");
            KvitteringType mimeResponse = soapClient.opprettJPStatus(mimeInput);
            ResponseValidator.validateJPStatusResponse(mimeResponse);
            log.info("GeoIntegrationEndpoint::nyJPStatusHandler - End");
            return OppJournalpostStatus.oppJournalpostStatusResToSuFa(mimeResponse);

        } catch (ValidationException e) {
            log.error("nyJPStatusHandler - Validation error: Endpoint = {}, FieldName= {}, Message= {}, Stacktrace={}", e.getEndpointName(), e.getFieldName(), e.getMessage(), e.getStackTrace());
//            log.error(String.valueOf(SplunkLogging.log().med("Validation Error","nyJPStatusHandler")));
            throw new RuntimeException(e);
        } catch (DataMappingException e) {
            log.error("nyJPStatusHandler - Data mapping error: FieldName = {}, FieldValue= {}, Message= {}, Stacktrace={}", e.getFieldName(), e.getFieldValue(), e.getMessage(), e.getStackTrace());
//            log.error(String.valueOf(SplunkLogging.log().med("Data mapping error","nyJPStatusHandler")));
            throw new RuntimeException(e);

        }
    }

}

