package no.integration.svv.utils.authentication;

import io.jsonwebtoken.*;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.client.RestTemplate;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPublicKeySpec;
import java.util.Base64;
import java.util.List;
import java.util.Objects;

import static no.integration.svv.utils.authentication.AuthenticationConstants.*;


public class Authentication {

    private static final Logger log = LoggerFactory.getLogger(Authentication.class);

    PublicKey publicKey;

    public void validate(String jwsToken, List<String> roles, String jwks, String oidcIssuer) throws NoSuchAlgorithmException, InvalidKeySpecException, ExpiredTokenException, MissingTokenException, MissingRoleException, InvalidTokenException, JSONException {
        log.info("Authentication::validate - Validating token and role...");

        if (jwsToken == null || jwsToken.isEmpty()) {
            log.error("Authentication::validate {}", AUTH_TOKEN_MISSING_LOGTEKST);
            throw new MissingTokenException(AUTH_TOKEN_MISSING_LOGTEKST);
        }
        Jws<Claims> jws = getJwsCached(jwsToken, jwks, oidcIssuer);


        final Object svvroles = jws.getBody().get("svvroles");

        if (!checkValidRole(roles, svvroles)) {
            log.error("Authentication::validate {}", AUTH_ROLE_MISSING_LOGTEKST);
            throw new MissingRoleException(AUTH_ROLE_MISSING_LOGTEKST);

        }

        Object tokenName = jws.getBody().get("tokenName");
        if (tokenName != null && !tokenName.toString().equals("id_token")) {
            log.error("Authentication::validate {}", AUTH_INVALID_NOT_ID_TOKEN_LOGTEKST);
            throw new InvalidTokenException(AUTH_INVALID_NOT_ID_TOKEN_LOGTEKST);
        }
        log.info("Authentication::validate - Token and role validation completed successfully");
    }

    boolean checkValidRole(List<String> roles, Object svvroles) {
        if (svvroles == null) {
            return false;
        }
        boolean valid = false;
        for (String role : roles) {
            if (svvroles.toString().contains(role)) {
                valid = true;
                break;
            }
        }
        return valid;
    }

    public Jws<Claims> getJwsCached(String jwsToken, String jwks, String oidcIssuer) throws InvalidKeySpecException, ExpiredTokenException, NoSuchAlgorithmException, InvalidTokenException, JSONException {

        /* Initielt kall til cache må opprette publicKey */

        if (publicKey == null) {
            JSONObject jwtHeader = new JSONObject(new String(Base64.getUrlDecoder().decode(jwsToken.split("\\.")[0])));
            publicKey = getPublicKey(jwks, jwtHeader);
        }

        Jws<Claims> jws;
        try {
            jws = Jwts.parser()
                    .requireIssuer(oidcIssuer)
                    .setSigningKey(publicKey)
                    .parseClaimsJws(jwsToken);
        } catch (ExpiredJwtException e) {
            log.error("Authentication:: getJwsCached{}", AUTH_TOKEN_EXPIRE_LOGTEKST);
            throw new ExpiredTokenException(AUTH_TOKEN_EXPIRE_LOGTEKST);
        } catch (Exception e) {
            log.error("Authentication:: getJwsCached{}", AUTH_CACHED_FAILED_LOGTEKST);
            log.error(AUTH_CACHED_FAILED_LOGTEKST);
            jws = getJws(jwsToken, jwks, oidcIssuer);
        }
        return jws;
    }

    public Jws<Claims> getJws(String jwsToken, String jwks, String oidcIssuer) throws ExpiredTokenException, NoSuchAlgorithmException, InvalidKeySpecException, InvalidTokenException, JSONException {

        /* Oppfrisk publicKey cache, da kall til getJwsCached feilet */
        JSONObject jwtHeader = new JSONObject(new String(Base64.getUrlDecoder().decode(jwsToken.split("\\.")[0])));
        publicKey = getPublicKey(jwks, jwtHeader);

        Jws<Claims> jws;
        try {
            jws = Jwts.parser()
                    .requireIssuer(oidcIssuer)
                    .setSigningKey(publicKey)
                    .parseClaimsJws(jwsToken);
        } catch (ExpiredJwtException e) {
            log.error("Authentication:: getJws{}",AUTH_TOKEN_EXPIRE_LOGTEKST);
            throw new ExpiredTokenException(AUTH_TOKEN_EXPIRE_LOGTEKST);
        } catch (InvalidClaimException e) {
            log.error("Authentication:: getJws{}",AUTH_INCORRECT_CLAIM_LOGTEKST);
            throw new InvalidTokenException(AUTH_INCORRECT_CLAIM_LOGTEKST + ": " + e.getMessage());
        }
        return jws;
    }

    public PublicKey getPublicKey(String jwks, JSONObject jwtHeader) throws NoSuchAlgorithmException, InvalidKeySpecException, JSONException, InvalidTokenException {

        Keys keys = new RestTemplate().getForObject(jwks, Keys.class);
        Key key = null;

        for (int i = 0; i < Objects.requireNonNull(keys).getKeys().size(); i++) {
            if (keys.getKeys().get(i).getKid().equals(jwtHeader.getString("kid"))) {
                key = Objects.requireNonNull(keys).getKeys().get(i);
            }
        }
        if (key == null) {
            log.error("Authentication:: getPublicKey{}",AUTH_INVALID_TOKEN_LOGTEKST);
            throw new InvalidTokenException(AUTH_INVALID_TOKEN_LOGTEKST);
        }
        RSAPublicKeySpec spec = getRSAPublicKeySpec(Base64.getUrlDecoder().decode(key.getN()), Base64.getUrlDecoder().decode(key.getE()));
        return KeyFactory.getInstance("RSA").generatePublic(spec);
    }

    RSAPublicKeySpec getRSAPublicKeySpec(byte[] modulus, byte[] exponent) {
        // Unsigned RSA modulus must be a power of 2 - in other words 128, 256, 512 [and so on] bytes.
        if (modulus.length % 2 == 0) {
            return new RSAPublicKeySpec(new BigInteger(1, modulus), new BigInteger(1, exponent));
        } else {
            return new RSAPublicKeySpec(new BigInteger(modulus), new BigInteger(exponent));
        }
    }
}